library(mclust)

### comment out the next line if the data are already read in
source("readpaulia.R")

### Change the next lines as appropriate
setsize <- 2000
trset <- sample(npixels, setsize, replace=FALSE)
minclus <- 3
maxclus <- 15
models <- .Mclust$emModelNames
.Mclust$itmax <- c(100, Inf)
nbest <- 5 # number of models to consider for extra EM steps

### Go!
X <- pict[pixels[trset], ]
bicvals <- EMclust(X, G=minclus:maxclus, emModelNames=models)
G <- minclus:maxclus
### dont be bothered by NA's (unstable clusterings)
bicvals[is.na(bicvals)] <- min(bicvals, na.rm=TRUE) - 10

### Find nbest best models (highest BIC values)
bestones <- rev(order(bicvals))[1:nbest]
bestG <- bestones %% length(G)
bestG[bestG == 0] <-  length(G)
bestG <- G[bestG]
bestmodels <- models[ceiling(bestones / length(G))]

znews <- vector(mode="list", nbest)
for (k in 1:nbest) {
  sumry <- summary(bicvals, X, G=bestG[k], modelNames=bestmodels[k])
  znews[[k]] <- do.call("em", c(list(data=pict[pixels,]), sumry))
}

### Final classifications are judged on likelihood
llks <- sapply(znews, function(x) x$loglik)
bestone <- which(llks == max(llks, na.rm=TRUE))[1] # in case of ties

classif <- map(znews[[bestone]]$z)
